(* 
 *
 * Copyright (c) 2003-2006, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 *)



module H = Hashtbl
 
type pmt = string;;
type pmth = (pmt, unit) H.t;;

class partialMap (name: string) = object (self)

   val constraintStore : ((pmt * pmt), unit) H.t =  H.create 7
   val leftStore :  (pmt, pmth) H.t = H.create 7
   val rightStore : (pmt, pmth) H.t = H.create 7

   method private addCorrespondent (h : (pmt, pmth) H.t) (a : pmt) (b : pmt) : unit = 
      try
         let ch : pmth = H.find h a in 
         H.replace ch b () 

      with Not_found ->
         (let ch : pmth = H.create 3 in
          H.add ch b ();
          H.add h a ch)   

   method private getCorrespondents (h : (pmt, pmth) H.t) (a : pmt) : pmt list = 
      try
         let ch : pmth = H.find h a in
         Commonutil.hashtbl_keys ch 
      with Not_found ->
         [] 

   method getLRCorrespondents (a : pmt) : pmt list = 
      self#getCorrespondents leftStore a

   method getRLCorrespondents (b : pmt) : pmt list = 
      self#getCorrespondents rightStore b
         
   method private addConstraintToStore ((a, b) : (pmt * pmt)) : unit =
      Commonutil.hashtbl_add_once constraintStore (a,b) ();
      
   method addConstraint ((a, b) : (pmt * pmt)) =
      self#addConstraintToStore (a, b);
      self#addCorrespondent leftStore  a b;
      self#addCorrespondent rightStore b a;

   method getBijectiveSubmapping () = 
      let isBijectionConstraint (a,b) = 
         match self#getLRCorrespondents a, self#getRLCorrespondents b with
            [b], [a] -> true
          | _ -> false
      in   
      List.filter isBijectionConstraint (Commonutil.hashtbl_keys constraintStore)

   method getRenamings () = 
      let isRenaming (a,b) = 
         match self#getRLCorrespondents a, self#getLRCorrespondents b with
            [], [] -> true
          | _ -> false
      in   
      List.filter isRenaming (self#getBijectiveSubmapping ())

end;;


(* Standalone testing *)

(*let pm = new partialMap "";;
pm#addConstraint ("a","b");;
pm#addConstraint ("a","c");;
pm#addConstraint ("d","a");;
pm#addConstraint ("f","g");;
pm#addConstraint ("h","i");;
pm#addConstraint ("h","j");;
pm#getBijectiveSubmapping ();;
pm#getRenamings ();; *)

(* let pm = new partialMap "";; *)
(* pm#addConstraint ("diff","biff");; *)
(* pm#addConstraint ("diff","biff");; *)
(* List.iter (fun (x,y) -> Printf.printf "getRenamings :  %s %s\n" x y) (pm#getRenamings ());; *)
(* raise (Failure "");; *)
