(* 
 *
 * Copyright (c) 2004-2006, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 *)



(*
   In order to avoid a clash of function symbols when we load the dynamic library we version every 
   function by the mapping
      f |-> f__vn
   where n is the current version of this function; initially zero.

   This phase is done at the end of the compilation chain.
*)

open Cil

module H = Hashtbl

let shouldRename (fname : string) : bool = 
   not (
      (Namedtypes.isConAbsFuncName fname) || 
      (Updateconfig.isUpdateFunc fname) ||
(*       (Ttpragma.isTT fname) || *)
(*       (Ttwrapper.isTTWrapper fname) || *)
      (fname = "main") ||
      (fname = Updateconfig.empty_tt_name) ||
      (fname = Updateconfig.patch_install_function_name) ||
      (Updateconfig.isWrapperFunction fname )   ||
      (fname = Updateconfig.state_xform_function_name)
      )

(* Rename functions and construct the new <-> old mapping *)
let funVersioning_iter (vd : Versiondata.version_data) (versionedFunHash :  (string, string) H.t) (g : Cil.global) : unit = 
   match g with 
      GFun(fd,l) -> 
         let functionName = fd.svar.vname in 
         begin try 
            (* don't rename con/abs/update *)
            if shouldRename functionName then 
               begin
                  let functionVersion = Versiondata.getFunctionVersion vd functionName in
                  let newName = Updateconfig.makeVersionedFunctionName functionName functionVersion in
                  H.add versionedFunHash newName fd.svar.vname; 
                  fd.svar.vname <- newName
               end                 
         with 
            Not_found -> 
               Updatedebug.abort (Updatedebug.bug "funVersioning_iter: couldn't get version number of function %s\n" functionName)
         end
    | _ -> ()

let funVersioning (vd : Versiondata.version_data) (file : Cil.file) : (string, string) H.t =
   (* Patchsplit needs this from us, cos' it needs to know whether a function was renamed or not *)
   let versionedFunHash = H.create 127 in
   Cil.iterGlobals file (funVersioning_iter vd versionedFunHash);
   versionedFunHash;;   
