(* 
 *
 * Copyright (c) 2004-2006, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 *)



open Cil

module H = Hashtbl

(* Collect all intern definitions, e.g. the ones that are definined in the original program or
   added afterwards *)

let makeInternVarHash (file : Cil.file) (vd : Versiondata.version_data) : (string, unit) H.t =
   let res : (string, unit) H.t = H.create 4095 in
   let sweepInternalNames_iter g = 
      match g with
            GVar(vi, ii, loc) -> Commonutil.hashtbl_add_once res vi.vname ()
         |  GFun(fd, loc) -> Commonutil.hashtbl_add_once res fd.svar.vname ()
         |  GVarDecl (vi, loc) -> 
                  (* if we're in patch mode, don't forget the external decls *)
                     if Updateargs.args.Updateargs.arg_currentMode = Updateargs.PatchMode &&
                        (Versiondata.isExistingFunction vd vi.vname) ||
                        (Versiondata.isExistingVar vd vi.vname)
                     then
                        Commonutil.hashtbl_add_once res vi.vname ()
         | _ -> ()
   in
   List.iter sweepInternalNames_iter file.globals;
   res;;

