(* 
 *
 * Copyright (c) 2004-2006, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 *)



(*
  setjmp and longjmp have to be treated specially, since they act as non-local goto's
 
  For the case:

  f() {
  ...

  D       setjmp(env);         D'
  ...
  }


  g() {
  ...

  D1       longjmp(env, val);   D1'
  ...
  }

  we add the conditional constraint    (mu_g = U) => D' \subseteq D1' 

  This is just like a goto.
  
*)

open Cil
module H = Hashtbl


(* module SetjmpSubtypeActions =  *)
(* struct *)
(*   module M = SAMonad *)
(*   module MOps = Monad.MonadOps(SAMonad) *)

(*   (\* Called when SubtypeCmp performs a standard subtype step *\) *)
(*   let subtype (t1: Cil.typ) (t2: Cil.typ) : unit M.m = *)
(*     match t1,t2 with  *)
(*       | TFun(t1',args_opt1,isVA1,attr1), TFun(t2',args_opt2,isVA2,attr2) ->   *)
(*           addConstraint t2 attr2 t1 attr1 *)

(*       (\* Possible bug (may be fixed): missing annotation *\) *)
(*       | TPtr(t1',attr1), TPtr(t2',attr2) -> *)
(*           addConstraint t2 attr2 t1 attr1 *)
   
(*       | _ -> return () *)


(*   let physicalSubtype (t1: Cil.typ) (t2: Cil.typ) : unit M.m = return () *)
(*   let externalSubtype (t1: Cil.typ) (t2: Cil.typ) : unit M.m = return () *)
(*   let ptrIncompatible (t1: Cil.typ) (t2: Cil.typ) (msg: string) : unit M.m = return () *)
(*   let invalidCSubtype (t1: Cil.typ) (t2: Cil.typ) (msg: string) : unit M.m = return () *)
(*   let toVoidPtr t1 t2 = return () *)
(*   let fromVoidPtr t1 t2 = return () *)
(* end *)



let isSetjmpFunction name = List.mem name Updateconfig.predefinedSetjmp;;
let isLongjumpFunction name = List.mem name Updateconfig.predefinedLongjmp;;

(* env var -> statement id mapping *)
type setjmpHash_t = (Cil.varinfo, int) H.t


(* add the (env_var, sid) mapping if the instruction is a call to setjmp *)
class collectSetjmpInstrVisitor setjmpHash sid : cilVisitor = object(self)
   inherit nopCilVisitor
      
   method vinst i = match i with
         Call(_, Lval(Var vi, NoOffset), e :: _, _) when isSetjmpFunction vi.vname ->
            let bareVarinfo = Commoncilutil.getBaseVarinfo e in
            ignore (Updatedebug.log [Updatedebug.BTAlways] "collectSetjmpInstrVisitor : %a -> %s, sid %d\n" 
                       d_plainexp e 
                       bareVarinfo.vname
                       sid);   
            H.add setjmpHash bareVarinfo sid;
            SkipChildren

      | _ -> SkipChildren
end


class collectSetjmpStmtVisitor setjmpHash : cilVisitor = object(self)
   inherit nopCilVisitor

   method vstmt s = match s.skind with
        Instr il -> 
           let instrVisitor = new collectSetjmpInstrVisitor setjmpHash s.sid in
           List.iter (fun i -> ignore (Cil.visitCilInstr instrVisitor i)) il;
           DoChildren

      | _ -> DoChildren
end

(* Collect stmt id's for all setjmp instructions *)
let collectSetjmpPoints (file : Cil.file) : setjmpHash_t =
   let setjmpHash : setjmpHash_t = H.create 7 in 
    
   (* then take care of the programmer-inserted ones *)   
   Cil.visitCilFileSameGlobals (new collectSetjmpStmtVisitor setjmpHash) file ;
   setjmpHash
