(* 
 *
 * Copyright (c) 2004-2006, 
 * 
 *  Iulian Neamtiu      <neamtiu@cs.umd.edu> 
 *  Michael Hicks       <mwh@cs.umd.edu> 
 *  Gareth Stoyle       <gareth.stoyle@cl.cam.ac.uk> 
 *
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met: 
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * 3. The names of the contributors may not be used to endorse or promote 
 * products derived from this software without specific prior written 
 * permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 *
 *
 *)



type mode = TestMode | NormalMode | PatchMode | PrintVDMode
type updatePointsPolicy = UNone | UReturnonly | UFull
type effectPrintingPolicy = EPNone | EPTypes | EPFunctions | EPFull
type unsoundnessAction = UnsoundWarn | UnsoundIgnore | UnsoundKillTypeShallow | UnsoundKillTypeDeep | UnsoundDie 
type typeTransformersGeneration = TTGYes | TTGNo | TTGExclusive
type args = 
   {
      mutable arg_currentMode           : mode;                 (* compilation mode *)
      mutable arg_updatePointsPolicy    : updatePointsPolicy;   (* update() keyword insertion policy *)
      mutable arg_effectPrintingPolicy  : effectPrintingPolicy; (* print effects policy *)
      mutable arg_patchDirectoryOut     : string;               (* directory where patches are stored *)
      mutable arg_versionDataInFile     : string;               (* file name to retrieve version data from *)
      mutable arg_versionDataOutFile    : string;               (* file name to store version data from *)
      mutable arg_dosuspect             : bool;                 (* whether or not to do suspect alias analysis *)
      mutable arg_dostaticconabs        : bool;                 (* whether to make con/abs functions static+inline *)
      mutable arg_dogvartypes           : bool;                 (* whether or not to do type introduction for gvars *)
      mutable arg_dopatchsplitting      : bool;                 (* split patches; by default is on *)
      mutable arg_gentts                : typeTransformersGeneration; (* generate type transformers; default is on (TTGYes) *)
      mutable arg_stoploops             : bool;                 (* emit code after extracting loops *)
      mutable arg_assembly              : unsoundnessAction;    (* what to do with inline assembly *)
      mutable arg_fromvoid              : unsoundnessAction;    (* what to do with casts from void* *)
      mutable arg_subtypes              : unsoundnessAction;    (* what to do with attempted physical or width subtypes under pointers *)
      mutable arg_restrictFuns          : bool;                 (* add functions used in continuation to restriction on __DSU_update() *) 
      mutable arg_restrictGvars         : bool;                 (* add global variables used in continuation to restriction on __DSU_update() *)
      mutable arg_restrictTypes         : bool;                 (* add types used in continuation to restriction on __DSU_update() *)

   }

(* Command line arguments stored here *)
let args = 
   { 
      (* These are the defaults *)
      arg_currentMode = NormalMode;
      arg_updatePointsPolicy = UReturnonly;
      arg_effectPrintingPolicy = EPTypes;
      arg_patchDirectoryOut = "";
      arg_versionDataInFile = "";
      arg_versionDataOutFile = "";
      arg_dosuspect = true;
      arg_dostaticconabs = true;
      arg_dogvartypes = false;
      arg_dopatchsplitting = true;
      arg_gentts = TTGYes;
      arg_stoploops = false;
      arg_assembly = UnsoundWarn;
      arg_fromvoid = UnsoundKillTypeShallow;
      arg_subtypes = UnsoundWarn;
      arg_restrictFuns = false;
      arg_restrictGvars = false;
      arg_restrictTypes = true;
   }

