/* Patch generation test suite 
 *
 * Changes: added extra field (y) to elements stored in list
 */

#pragma __DSU_malloc("malloc");

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>


/********************************************************************
* Items to store in list                                            *
*********************************************************************/
struct T {
	int x;
   int y;   /* new field */
};

void printT(struct T* pt) 
{
   printf("t.x=%d, t.y=%d\n",pt->x, pt->y);
   return;
}



/********************************************************************
* List definition and functions                                     *
*********************************************************************/
struct Tlist {
  struct T item;
  struct Tlist* next;
};

struct Tlist* consT(struct T* pt, struct Tlist* pl)
{
  struct Tlist* l = malloc(sizeof(struct Tlist));
  l->next = pl;
  l->item = *pt;

  return l;
}

void iterT(void(*f)(struct T*), struct Tlist* pl)
{
  while(pl)
    {
      f(&pl->item);
      pl = pl->next;
    }
  return;
}

extern int update_requested;

/********************************************************************
* The main program loop will be extracted                                            *
*********************************************************************/
#pragma __DSU_loop("MAINLOOP")

void interactive_update(struct Tlist* pl) 
{
   printf("*** interactive_update version 1: list elements have two members, .x and .y\n");
   iterT(printT,pl);

  /* Do an update */
  printf("*** interactive_update version 1: Hit ENTER to perform update\n");
  getchar();
  printf("*** Start update\n");
  /* Usually signalling an available update is triggered by sending a 
     SIGUSR2 to the process, but in this case we want to spare the user of doing that */
  update_requested = 1;
  __DSU_update("");
  printf("*** End update \n");
}

int global_version = 1;

int main()
{
  struct Tlist* pl = 0;
  struct T t;
  int i;
  
  /* Create a list */
  for(i=10; i>0; i--) {
    t.x = i;
    pl = consT(&t,pl);
  }

  i = 0;

  MAINLOOP:while(1) {  
      printf("*** MAINLOOP: main() version 1, global_version = %d\n", global_version);

      interactive_update(pl);
  
      i++;
      if (i >= 3)
        break;
   }
  return 0;
}

