#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>



/*
 * Structure for all the elements of the list.
 */
struct __DSU_FVersionElem {
  int vNumber;
  int nRef;
  char *libName;
  void *fct;
  void *handler;
  struct __DSU_FVersionElem* next;
  struct __DSU_FVersionElem* prev;
};

/*
 * Creates a __DSU_FVersionElem
 */
struct __DSU_FVersionElem* __DSU_consFVersionElem(char *libName, void *handler, int vNumber, void *fct);

/*
 * Structure for the head of the list.
 */ 
struct __DSU_FVersionList{
  char *baseFunName;
  struct __DSU_FVersionElem *first;
  void *activeFunction;
  unsigned int oldVersionsTotalCount;
  unsigned int length;
};


/*
 * Creates an empty __DSU_FVersionList
 */
struct __DSU_FVersionList* __DSU_consFVersionList(char *baseFunName);

/*
 * adds an element at the end of the linkedlist
 * MO: NO NEED TO BE THREAD-SAFE, AT UPDATE-TIME
 * we add it at the beginning for the reason that it is likely that
 * most recent versions will probably be used more often.
 */
void __DSU_addFVersionList(struct __DSU_FVersionList *l,char *name, unsigned int f_versionNumber, unsigned int p_versionNumber);


/*
 * Prints the state of a version list on stderr.
 * MO: SHOULD BE THREAD-SAFE
 */
void __DSU_printFVersionList(struct __DSU_FVersionList *l);

/*
 * List for functions to be GCed
 */
struct __DSU_Fct2Clean{
  struct __DSU_FVersionList *l;
  struct __DSU_Fct2Clean *next;

};

/*
 * add to a __DSU_Fct2Clean
 */
void __DSU_addFct2Clean(struct __DSU_Fct2Clean *cleaner,struct __DSU_FVersionList *l);

/*
 * Variable used to see if a GC has to be triggered.
 */
extern int __DSU_TRIGGER_FGC; 
extern struct __DSU_Fct2Clean __DSU_FGARBAGE;

/*
 * this function makes the effective garbage collection
 */
void __DSU_FGC();

/*
 * This loads the function.
 * returns:
 *    -1 if it was a failure.
 *    0 otherwise.
 */
int __DSU_load(const char *libname, unsigned int versionNumber);
/*
 * This unloads the old function.
 * returns:
 *    -1 if it was a failure.
 *    0 if the unloading was effectively made.
 */
int __DSU_unload(struct __DSU_FVersionList *l, unsigned int versionNumber);
/*
 * This locks the function.
 * MO: SHOULD BE THREAD-SAFE
 * returns:
 *    -1 if it was a failure.
 *    n: number of locks taken on this function (including this one).
 */
int __DSU_lock(struct __DSU_FVersionList *l, unsigned int versionNumber);

/*
 * This locks the function.
 * MO: SHOULD BE THREAD-SAFE
 * returns:
 *    -1 if it was a failure.
 *    n: number of locks still taken on this function (after this one is released).
 */
int __DSU_unlock(struct __DSU_FVersionList *l, unsigned int versionNumber);
