#include <pthread.h>
#include <stdio.h>
#include "mt.h"

#define SLEEP_NANOSEC 50000000

static int barrier = 0;
static int thread_count = 0;
pthread_mutex_t count_mtx;
pthread_mutex_t bar_mtx;

void __DSU_mt_init()
{
  pthread_mutex_init(&count_mtx, NULL);
  pthread_mutex_init(&bar_mtx, NULL);
}


void thread_register(const char * s)
{
  pthread_mutex_lock(&count_mtx);
  thread_count++;
  pthread_mutex_unlock(&count_mtx);

  pthread_mutex_lock(&bar_mtx);
  barrier++;
  pthread_mutex_unlock(&bar_mtx);

  DSU_DEBUG("thread '%s' (%x) registered\n", s, pthread_self());
}

void thread_deregister(const char * s)
{
  pthread_mutex_lock(&count_mtx);
  thread_count--;
  pthread_mutex_unlock(&count_mtx);

  pthread_mutex_lock(&bar_mtx);
  barrier--;
  pthread_mutex_unlock(&bar_mtx);

  DSU_DEBUG("thread '%s' (%x) deregistered\n", s, pthread_self());
}


void thread_check_in()
{
  struct timespec ts;
  ts.tv_sec = 0;
  ts.tv_nsec = SLEEP_NANOSEC;

  pthread_mutex_lock(&bar_mtx);
  barrier--;

  if (barrier == 0) {
    DSU_DEBUG("************ THREADS SYNCHRONIZED (check_in) id=%x *****************\n", pthread_self());
  }
  else {
    DSU_DEBUG("...no luck synchronizing (check_in)...\n");
  }
  pthread_mutex_unlock(&bar_mtx);


  //nanosleep((struct timespec  const  *)(& ts), (struct timespec *)((void *)0));
}

void thread_check_out()
{
  struct timespec ts;
  ts.tv_sec = 0;
  ts.tv_nsec = SLEEP_NANOSEC;

  //nanosleep((struct timespec  const  *)(& ts), (struct timespec *)((void *)0));

  pthread_mutex_lock(&bar_mtx);
  barrier++;

  if (barrier == 1) {
    DSU_DEBUG("************ THREADS SYNCHRONIZED (check_out) id=%x *****************\n", pthread_self());
  }
  else {
    DSU_DEBUG("...no luck synchronizing (check_out)...\n");
  }
  pthread_mutex_unlock(&bar_mtx);
}

int threads_synchronized_test_set()
{
  int barrier_val;

  pthread_mutex_lock(&bar_mtx);

  barrier_val = barrier;
  if (barrier == 0) {
    DSU_DEBUG("************ THREADS SYNCHRONIZED (threads_synchronized) id=%x *****************\n", pthread_self());

    return THREADS_SYNCED;
  }
  else {
    DSU_DEBUG("...no luck synchronizing (threads_synchronized_ts)...\n");
    pthread_mutex_unlock(&bar_mtx);
  }

  return barrier_val;
}

void threads_synchronized_release()
{
  pthread_mutex_unlock(&bar_mtx);
}
