#ifndef MT_H
#include "dsu_debug.h"

#define THREADS_SYNCED 0

/* DSU multithreading (MT) API */

/* Initializes MT API. Called by __DSU_init_handler()  */
extern void __DSU_mt_init();

/* a thread calls thread_check_in() to signal it's entering a quiescent
 * region, i.e., it's cool with me to do an update
 *
 * thread_check_out() means a thread is exiting its quiescent region
 *
 * an update can only be performed when all threads are checked in
 */
extern void thread_check_in();
extern void thread_check_out();

/*
 * thread_register() tells the MT API about the existence of this thread  
 * thread_deregister() tells the MT API the current thread doesn't exist any more

 * these functions increment/decrement the thread count for quiescence
 */
extern void thread_register(const char * s);
extern void thread_deregister(const char * s);

/* 
 * internal functions  
 */
extern int  threads_synchronized_test_set();
extern void threads_synchronized_release();
#endif  //MT_H
