#include <string.h>

struct patch_metadata
{
  char ** forbidden_functions; 
};

struct patch_metadata metadata_array[100];

static int current_version = 0;


#define MAXTOKENS  4096

/* split string into tokens, return token array */
char **split(char *string, char *delim) {
  char **tokens = NULL;
  char *working = NULL;
  char *token = NULL;
  int idx = 0;

  tokens  = malloc(sizeof(char *) * MAXTOKENS);
  if(tokens == NULL)
    return NULL;
  working = malloc(sizeof(char) * strlen(string) + 1);
  if(working == NULL)
    return NULL;

  /* to make sure, copy string to a safe place */
  strcpy(working, string);
  for(idx = 0; idx < MAXTOKENS; idx++)
    tokens[idx] = NULL;

  token = strtok(working, delim);
  idx = 0;

  /* always keep the last entry NULL termindated */
  while((idx < (MAXTOKENS - 1)) && (token != NULL)) {
    tokens[idx] = malloc(sizeof(char) * strlen(token) + 1);
    if(tokens[idx] != NULL) {
      strcpy(tokens[idx], token);
      idx++;
      token = strtok(NULL, delim);
    }
  }

  free(working);
  return tokens;
}

static int is_present(char * fun_name)
{
  int i;
  char **tokens = metadata_array[current_version].forbidden_functions;

  for(i = 0; tokens[i]; i++)
    {
      if (strcmp(fun_name, tokens[i]) == 0)
        return 1;
    }  
  
  /* not found */
  return 0;
}

void new_metadata(char * forbidden_functions, int version)
{
  char *delim = " ";
  char **tokens = NULL;

  /* this makes deep cpoies, so don't bother copying forbidden_functions */
  tokens = split(forbidden_functions, delim);

  current_version = version;
  metadata_array[version].forbidden_functions = tokens;
}

/* Return 0 if sets not disjoint, 1 otherwise */
int is_allowed_update(char * modified_functions)
{
  int i;
  char *delim = " ";
  char **tokens = NULL;
  char * token_iterator;

  /* this makes deep copies, so don't bother copying forbidden_functions */
  tokens = split(modified_functions, delim);

  for(i = 0; tokens[i]; i++)
    {
      if (is_present(tokens[i]))
        /* found, not_allowed */
        return 0;
    }

  /* no one found, update allowed */
  return 1;
 
}

/* code below is for testing purposes only */

/*
int expect (int cond, int test_nr)
{
  if (cond)
    printf("Test %d passed\n", test_nr);
  else
    printf("Test %d failed\n", test_nr);
}


typedef char * pchar;
int main(int argc, char ** argv)
{
  int version;
  pchar forbidden[] = {" foo bar baz", " foo bar baz", " foo bar baz"};
  pchar modified[] = {" innie minnie miny ", " foo ", " "};
  int expected_results[] = {1,0,1};


  for (version = 0; version < 3; version++ )
    {
      new_metadata(forbidden[version], version);
      expect(is_allowed_update(modified[version]) == expected_results[version], version);
    }

  return 0;
  
}
*/
